/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.container;

import org.netkernel.request.IRequest;
import org.netkernel.request.IResponse;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.ISpace;

/** An external listener which is notified when various kernel events happen
 * @author tab
 */
public interface IKernelListener
{
	void externalRequest(IRequest aRootRequest, Object aTransportInstance, ISpace aTransportSpace);
	
	/** Called when an endpoint is about to be invoked to service a request
	 * @param aRootRequest the root request which initiated this sequence of requests
	 * @param aParent the immediate parent request of the request to be serviced
	 * @param aRequestIn the request to be serviced
	 * @param aEndpoint the endpoint to receive the request 
	 */
	void endpointInvoke(IRequest aRootRequest, IRequest aParent, IRequest aRequestIn, IRequest aRequestOut, IEndpoint aEndpoint);
	
	/** Called after the invocation of an resource endpoint
	 * @param aRootRequest the root request which initiated this sequence of requests
	 * @param aRequestIn the request to be serviced
	 * @param aEndpoint the endpoint to receive the request 
	 * @param aDuration the computed execution time of the endpoint - this is not elapsed time
	 * @param aResponse the response generated by the endpoint
	 */
	void endpointResponse(IRequest aRootRequest, IRequest aRequestIn, IEndpoint aEndpoint, long aDuration, IResponse aResponse);
	
	/** Called when the cache finds a suitable response to a request
	 * @param aRootRequest the root request which initiated this sequence of requests
	 * @param aParent the immediate parent request of the request to be serviced
	 * @param aRequest the request to be serviced
	 * @param aResponse the response from the cache
	 */
	void cacheHit(IRequest aRootRequest, IRequest aParent, IRequest aRequest,  IResponse aResponse);
	
	
	/** Called when the resolution of a request fails
	 * @param aRootRequest the root request which initiated this sequence of requests
	 * @param aParent the immediate parent request of the request to be serviced
	 * @param aRequest the request to be serviced
	 * @param aResponse the response containing a ResolutionFailureException
	 */
	void resolutionFailure(IRequest aRootRequest, IRequest aParent, IRequest aRequest, IResponse aResponse);
}
