/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request.impl;

import org.netkernel.request.IExpiry;
import org.netkernel.request.IRequest;
import org.netkernel.util.Utils;
import org.netkernel.util.XMLUtils;

public class ExpiryFactory
{
	public static final int EXPIRY_ALWAYS=0;
	public static final int EXPIRY_CONSTANT=1;
	public static final int EXPIRY_NEVER=2;
	public static final int EXPIRY_DEPENDENT=3;
	public static final int EXPIRY_MIN_CONSTANT_DEPENDENT=4;
	public static final int EXPIRY_MAX_CONSTANT_DEPENDENT=5;
	public static final int EXPIRY_FUNCTION=6;	
	public static final int EXPIRY_MIN_FUNCTION_DEPENDENT=7;	
	
	private static final IExpiry sAlwaysExpired = new IExpiryInternal()
	{	
		public String toString()
		{	return "AlwaysExpired";
		}

		public int getType()
		{	return EXPIRY_ALWAYS;
		}

		public boolean isExpired(long aNow)
		{	return true;
		}
	};
	
	private static final IExpiry sNeverExpired = new IExpiryInternal()
	{	public boolean isExpired(long aNow)
		{	return false;
		}
		public String toString()
		{	return "NeverExpired";
		}
		public int getType()
		{	return EXPIRY_NEVER;
		}
	};	
	
	private static final IExpiry sDependentExpiry = new IExpiryInternal()
	{	
		public String toString()
		{	return "DependentExpiry";
		}
		public boolean tracksDependencies()
		{	return true;
		}
		public boolean isExpired(long aNow)
		{	return false;
		}		
		public int getType()
		{	return EXPIRY_DEPENDENT;
		}
	};
	
	
	public static IExpiry getAlwaysExpired()
	{	return sAlwaysExpired;
	}

	public static IExpiry getNeverExpired()
	{	return sNeverExpired;
	}
	
	public static IExpiry getDependentExpiry()
	{	return sDependentExpiry;
	}
	
	public static IExpiry getTimedExpiry(final long aExpireTime)
	{	return new IExpiryInternal()
		{	
			public boolean isExpired(long aNow)
			{	return aExpireTime<=aNow;
			}
			public String toString()
			{	return "TimedExpiry("+Utils.expiryToString(aExpireTime-System.currentTimeMillis())+")";
			}
			public int getType()
			{	return EXPIRY_CONSTANT;
			}
			public long getTime()
			{	return aExpireTime;
			}
		};
	}
	
	public static IExpiry getMinConstantDependentExpiry(final long aExpireTime)
	{	return new IExpiryInternal()
		{	
			public String toString()
			{	return "MinConstantDependentExpiry("+Utils.expiryToString(aExpireTime-System.currentTimeMillis())+")";
			}
			public boolean tracksDependencies()
			{	return true;
			}
			public int getType()
			{	return EXPIRY_MIN_CONSTANT_DEPENDENT;
			}
			public long getTime()
			{	return aExpireTime;
			}
		};
	}
	
	public static IExpiry getMaxConstantDependentExpiry(final long aExpireTime)
	{	return new IExpiryInternal()
		{	
			public String toString()
			{	return "MaxConstantDependentExpiry("+Utils.expiryToString(aExpireTime-System.currentTimeMillis())+")";
			}
			public boolean tracksDependencies()
			{	return true;
			}
			public int getType()
			{	return EXPIRY_MAX_CONSTANT_DEPENDENT;
			}
			public long getTime()
			{	return aExpireTime;
			}
		};
	}
	
	public static IExpiry getMinFunctionDependentExpiry(final IExpiry aFunction)
	{	return new IExpiryInternal()
		{	
			public String toString()
			{	return "MinFunctionDependentExpiry("+aFunction.toString()+")";
			}
			public boolean tracksDependencies()
			{	return true;
			}
			public int getType()
			{	return EXPIRY_MIN_FUNCTION_DEPENDENT;
			}
			public IExpiry getFunction()
			{	return aFunction;
			}
		};
	}
	
	
	public static IExpiry getDependentExpiry(final IExpiry[] aDeterminants)
	{	return new IExpiryInternal()
		{
			public boolean tracksDependencies()
			{	return true;
			}
			
			public String toString()
			{	return "DependentExpiry";
			}
			
			public int getType()
			{	return EXPIRY_DEPENDENT;
			}

			public boolean isExpired(long aNow)
			{	boolean expired=false;
				for (int i=aDeterminants.length-1; i>=0 && !expired; i--)
				{	expired=aDeterminants[i].isExpired(aNow);
				}
				return expired;
			}
			
			public IExpiry[] getExpiryDeterminants()
			{	return aDeterminants;
			}
		};
	}
	
	public static IExpiry getMinConstantDependentExpiry(final long aTime, final IExpiry[] aDeterminants)
	{	return new IExpiryInternal()
		{
			public boolean tracksDependencies()
			{	return true;
			}
			
			public String toString()
			{	return "MinConstantDependentExpiry("+Utils.expiryToString(aTime-System.currentTimeMillis())+")";
			}
			
			public int getType()
			{	return EXPIRY_MIN_CONSTANT_DEPENDENT;
			}

			public boolean isExpired(long aNow)
			{	boolean expired=aNow>aTime;
				if (!expired)
				{	for (int i=aDeterminants.length-1; i>=0 && !expired; i--)
					{	expired=aDeterminants[i].isExpired(aNow);
					}
				}
				return expired;
			}
			
			public IExpiry[] getExpiryDeterminants()
			{	return aDeterminants;
			}
		};
	}
	
	public static IExpiry getMaxConstantDependentExpiry(final long aTime, final IExpiry[] aDeterminants)
	{	return new IExpiryInternal()
		{
			public boolean tracksDependencies()
			{	return true;
			}
			
			public String toString()
			{	return "MaxConstantDependentExpiry("+Utils.expiryToString(aTime-System.currentTimeMillis())+")";
			}
			
			public int getType()
			{	return EXPIRY_MAX_CONSTANT_DEPENDENT;
			}

			public boolean isExpired(long aNow)
			{	boolean expired=aNow>aTime;
				if (expired)
				{	expired=false;
					for (int i=aDeterminants.length-1; i>=0 && !expired; i--)
					{	expired|=aDeterminants[i].isExpired(aNow);
					}
				}
				return expired;
			}
			
			public IExpiry[] getExpiryDeterminants()
			{	return aDeterminants;
			}
		};		
	}
	
	public static IExpiry getMinFunctionDependentExpiry(final IExpiry aFunction, final IExpiry[] aDeterminants)
	{	return new IExpiryInternal()
		{
			public boolean tracksDependencies()
			{	return true;
			}
			
			public String toString()
			{	return "MinFunctionDependentExpiry("+aFunction.toString()+")";
			}
			
			public int getType()
			{	return EXPIRY_MIN_FUNCTION_DEPENDENT;
			}

			public boolean isExpired(long aNow)
			{	boolean expired=aFunction.isExpired(aNow);
				if (!expired)
				{	for (int i=aDeterminants.length-1; i>=0 && !expired; i--)
					{	expired=aDeterminants[i].isExpired(aNow);
					}
				}
				return expired;
			}
			
			public IExpiry[] getExpiryDeterminants()
			{	return aDeterminants;
			}
			
			public IExpiry getFunction()
			{	return aFunction;
			}
		};
	}
	
	public abstract static class IExpiryInternal implements IExpiry
	{	
		private static final IExpiry[] NO_DETERMINANTS=new IExpiry[0];
		
		public boolean tracksDependencies()
		{	return false;
		}
		public abstract int getType();
		public long getTime()
		{	return 0;
		}
		public IExpiry[] getExpiryDeterminants()
		{	return NO_DETERMINANTS;
		}
		public boolean isExpired(long aNow)
		{	return true;
		}
		public IExpiry getFunction()
		{	return null;
		}
	};
	
	public static boolean tracksDependencies(IExpiry aExpiry)
	{	boolean result;
		if (aExpiry instanceof IExpiryInternal)
		{	result = ((IExpiryInternal)aExpiry).tracksDependencies();
		}
		else
		{	result=false;
		}
		return result;
	}
	
	public static int getType(IExpiry aExpiry)
	{	int result;
		if (aExpiry instanceof IExpiryInternal)
		{	result = ((IExpiryInternal)aExpiry).getType();
		}
		else
		{	result=ExpiryFactory.EXPIRY_FUNCTION;
		}
		return result;
	}
	
	public static long getTime(IExpiry aExpiry)
	{	return ((IExpiryInternal)aExpiry).getTime();
	}
	
	public static IExpiry getFunction(IExpiry aExpiry)
	{	IExpiry result;
		if (aExpiry instanceof IExpiryInternal)
		{	result=((IExpiryInternal)aExpiry).getFunction();
		}
		else
		{	result=aExpiry;
		}
		return result;
	}
	
	public static IExpiry[] getExpiryDeterminants(IExpiry aExpiry)
	{	return ((IExpiryInternal)aExpiry).getExpiryDeterminants();
	}
	
	public static String toString(IExpiry aExpiry)
	{	StringBuilder sb = new StringBuilder(128);
		recurseToString(aExpiry,sb);
		return sb.toString();
	}
	
	private static void recurseToString(IExpiry aExpiry, StringBuilder aSB)
	{	
		aSB.append("<e msg=\"");
		aSB.append(XMLUtils.escape(aExpiry.toString())); 
		aSB.append("\">");
		
		if (aExpiry instanceof IExpiryInternal ) // && ((IExpiryInternal)aExpiry).tracksDependencies()
		{	for (IExpiry expiry: ((IExpiryInternal)aExpiry).getExpiryDeterminants())
			{	recurseToString(expiry,aSB);
			}
		}
		
		aSB.append("</e>");
	}		
	
	
	
}
