/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;
import org.netkernel.request.IRequest;

public class PriorityRequestTable
{
	private ConcurrentLinkedQueue<IRequestState> mQueueMin;
	private ConcurrentLinkedQueue<IRequestState> mQueueStd;
	private ConcurrentLinkedQueue<IRequestState> mQueueMax;
	private Semaphore mSem=new Semaphore(0,false);
	private volatile int mInterruptCount;
	
	public PriorityRequestTable()
	{	mQueueMin = new ConcurrentLinkedQueue<IRequestState>();
		mQueueStd = new ConcurrentLinkedQueue<IRequestState>();
		mQueueMax = new ConcurrentLinkedQueue<IRequestState>();
	}

	/** Puts a scheduler state into the table for future scheduling
	 * @param aState
	 */
	public void put(IRequestState aState)
	{	switch(aState.getRequestIn().getPriority())
		{	case IRequest.PRIORITY_MIN:
				mQueueMin.add(aState);
				break;
			case IRequest.PRIORITY_STD:
				mQueueStd.add(aState);
				break;
			case IRequest.PRIORITY_MAX:
				mQueueMax.add(aState);
				break;
		}
		mSem.release();
	}
	
	/**
     * Takes an item off the queue blocking indefinitely until something
     * is there unless interrupted.
     * @return The QueueItem taken from the queue
     * @exception InterruptedException thrown if we are interrupted whilst waiting for an item
     * either a real thread.interrupt or if this queue is interrupted
     */
	public IRequestState take() throws InterruptedException
	{	IRequestState result=null;
		int interruptCount=mInterruptCount;
		while (result==null)
		{	mSem.acquire();
			if (mInterruptCount!=interruptCount)
			{	throw new InterruptedException();
			}
			result=mQueueMax.poll();
			if (result==null)
			{	result=mQueueStd.poll();
				if (result==null)
				{	result=mQueueMin.poll();				
				}
			}
		}
		return result;
	}
	
	/** Causes all threads blocking on a take() call to the object to return
     * throwing an InterruptedException
     */    
    public void interrupt()
    {	mInterruptCount++;
    }
    
    private static final RequestState[] sNoStates = new RequestState[0];
    
    public RequestState[] dump()
    {	RequestState[] resultMax=mQueueMax.toArray(sNoStates);
    	RequestState[] resultStd=mQueueStd.toArray(sNoStates);
    	RequestState[] resultMin=mQueueMin.toArray(sNoStates);
    	RequestState[] result = new RequestState[resultMax.length+resultStd.length+resultMin.length];
    	System.arraycopy(resultMax, 0, result, 0, resultMax.length);
    	System.arraycopy(resultStd, 0, result, resultMax.length, resultStd.length);
    	System.arraycopy(resultMin, 0, result, resultMax.length+resultStd.length, resultMin.length);
    	return result;
    }
    
    public List<RequestState> clear(RootState aRoot)
    {	List<RequestState> result = new ArrayList<RequestState>();
    	ConcurrentLinkedQueue[] queues=new ConcurrentLinkedQueue[] { mQueueMax, mQueueStd, mQueueMin };
    	for (ConcurrentLinkedQueue<RequestState> queue : queues)
    	{ 	RequestState[] states=queue.toArray(sNoStates);
	    	for (RequestState state: states)
	    	{	if (state.getRoot()==aRoot)
	    		{	if (queue.remove(state))
	    			{ result.add(state);
	    			}
	    		}
	    	}
    	}
    	return result;
    }
}