/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.urii;

/**
 * Contains and manages a version number of the form "x(\.y)*".
 * This class is NOT thread safe.
 */
public interface IVersion
{
	 /**
     * Compares two versions.
     * The comparison is done the usual way, i.e.,  2.0 is greater than 1.99.1,
     * and 0.1.2 is greater than 0.1
     * @param aVersion the version to compare to.
     * @return 0 if this == aVersion, 1 if this greater than aVersion, -1 otherwise.
     */
	int compareVersions(IVersion aVersion);
	
	/**
     * Determine if this version is greater than the given one.
     * @param aVersion the version to compare to.
     * @return true if compareVersions(ver) &gt; 0
     * @see #compareVersions
     */
	boolean isGreaterThan(IVersion aVersion);

    /**
     * Determine if this version is greater than or equal to the given one.
     * @param aVersion the version to compare to.
     * @return true if compareVersions(ver) &gt;= 0
     * @see #compareVersions
     */
    boolean isGreaterOrEqualThan(IVersion aVersion);

    /**
     * Determine if this version is less than the given one.
     * @param aVersion the version to compare to.
     * @return true if compareVersions(ver) &lt; 0
     * @see #compareVersions
     */
    boolean isLessThan(IVersion aVersion);
 
    /**
     * Determine if this version is less than or equal to the given one.
     * @param aVersion the version to compare to.
     * @return true if compareVersions(ver) &lt;= 0
     * @see #compareVersions
     */
    boolean isLessOrEqualThan(IVersion aVersion);

}

